diff --git a/ntex-util/src/future/either.rs b/ntex-util/src/future/either.rs
index af59bfa7..2c142dfb 100644
--- a/ntex-util/src/future/either.rs
+++ b/ntex-util/src/future/either.rs
@@ -89,6 +89,12 @@ where
A: error::Error,
B: error::Error,
{
+ fn source(&self) -> Option<&(dyn error::Error + 'static)> {
+ match self {
+ Either::Left(a) => a.source(),
+ Either::Right(b) => b.source(),
+ }
+ }
}
impl fmt::Display for Either
diff --git a/ntex/CHANGES.md b/ntex/CHANGES.md
index 2961ebfa..642e4202 100644
--- a/ntex/CHANGES.md
+++ b/ntex/CHANGES.md
@@ -1,5 +1,11 @@
# Changes
+## [1.1.0] - 2024-02-07
+
+* http: Add http/1 control service
+
+* http: Add http/2 control service
+
## [1.0.0] - 2024-01-09
* web: Use async fn for Responder and Handler traits
diff --git a/ntex/Cargo.toml b/ntex/Cargo.toml
index 90799fae..1ab1107d 100644
--- a/ntex/Cargo.toml
+++ b/ntex/Cargo.toml
@@ -1,6 +1,6 @@
[package]
name = "ntex"
-version = "1.0.0"
+version = "1.1.0"
authors = ["ntex contributors "]
description = "Framework for composable network services"
readme = "README.md"
@@ -52,13 +52,13 @@ ntex-codec = "0.6.2"
ntex-connect = "1.0.0"
ntex-http = "0.1.12"
ntex-router = "0.5.3"
-ntex-service = "2.0.0"
+ntex-service = "2.0.1"
ntex-macros = "0.1.3"
-ntex-util = "1.0.0"
+ntex-util = "1.0.1"
ntex-bytes = "0.1.24"
ntex-h2 = "0.5.0"
ntex-rt = "0.4.11"
-ntex-io = "1.0.0"
+ntex-io = "1.0.1"
ntex-tls = "1.0.0"
ntex-tokio = { version = "0.4.0", optional = true }
ntex-glommio = { version = "0.4.0", optional = true }
diff --git a/ntex/src/http/body.rs b/ntex/src/http/body.rs
index 02d47234..048854e7 100644
--- a/ntex/src/http/body.rs
+++ b/ntex/src/http/body.rs
@@ -70,6 +70,15 @@ impl ResponseBody {
}
}
+impl From> for Body {
+ fn from(b: ResponseBody) -> Self {
+ match b {
+ ResponseBody::Body(b) => b,
+ ResponseBody::Other(b) => b,
+ }
+ }
+}
+
impl From for ResponseBody {
fn from(b: Body) -> Self {
ResponseBody::Other(b)
diff --git a/ntex/src/http/builder.rs b/ntex/src/http/builder.rs
index 859cdba5..14839efb 100644
--- a/ntex/src/http/builder.rs
+++ b/ntex/src/http/builder.rs
@@ -1,32 +1,31 @@
use std::{error::Error, fmt, marker::PhantomData};
-use ntex_h2::{self as h2};
-
use crate::http::body::MessageBody;
-use crate::http::config::{KeepAlive, OnRequest, ServiceConfig};
-use crate::http::error::ResponseError;
-use crate::http::h1::{Codec, ExpectHandler, H1Service, UpgradeHandler};
-use crate::http::h2::H2Service;
-use crate::http::request::Request;
-use crate::http::response::Response;
-use crate::http::service::HttpService;
-use crate::io::{Filter, Io, IoRef};
-use crate::service::{boxed, IntoService, IntoServiceFactory, Service, ServiceFactory};
-use crate::time::Seconds;
+use crate::http::config::{KeepAlive, ServiceConfig};
+use crate::http::error::{H2Error, ResponseError};
+use crate::http::h1::{self, H1Service};
+use crate::http::h2::{self, H2Service};
+use crate::http::{request::Request, response::Response, service::HttpService};
+use crate::service::{IntoServiceFactory, ServiceFactory};
+use crate::{io::Filter, time::Seconds};
/// A http service builder
///
/// This type can be used to construct an instance of `http service` through a
/// builder-like pattern.
-pub struct HttpServiceBuilder> {
+pub struct HttpServiceBuilder<
+ F,
+ S,
+ C1 = h1::DefaultControlService,
+ C2 = h2::DefaultControlService,
+> {
config: ServiceConfig,
- expect: X,
- upgrade: Option,
- on_request: Option,
+ h1_control: C1,
+ h2_control: C2,
_t: PhantomData<(F, S)>,
}
-impl HttpServiceBuilder> {
+impl HttpServiceBuilder {
/// Create instance of `ServiceConfigBuilder`
pub fn new() -> Self {
HttpServiceBuilder::with_config(ServiceConfig::default())
@@ -37,26 +36,25 @@ impl HttpServiceBuilder> {
pub fn with_config(config: ServiceConfig) -> Self {
HttpServiceBuilder {
config,
- expect: ExpectHandler,
- upgrade: None,
- on_request: None,
+ h1_control: h1::DefaultControlService,
+ h2_control: h2::DefaultControlService,
_t: PhantomData,
}
}
}
-impl HttpServiceBuilder
+impl HttpServiceBuilder
where
F: Filter,
S: ServiceFactory + 'static,
S::Error: ResponseError,
S::InitError: fmt::Debug,
- X: ServiceFactory + 'static,
- X::Error: ResponseError,
- X::InitError: fmt::Debug,
- U: ServiceFactory<(Request, Io, Codec), Response = ()> + 'static,
- U::Error: fmt::Display + Error,
- U::InitError: fmt::Debug,
+ C1: ServiceFactory, Response = h1::ControlAck>,
+ C1::Error: Error,
+ C1::InitError: fmt::Debug,
+ C2: ServiceFactory, Response = h2::ControlResult>,
+ C2::Error: Error,
+ C2::InitError: fmt::Debug,
{
/// Set server keep-alive setting.
///
@@ -138,70 +136,24 @@ where
self
}
- #[doc(hidden)]
- /// Configure http2 connection settings
- pub fn configure_http2(self, f: O) -> Self
+ /// Provide control service for http/1.
+ pub fn h1_control(self, control: CF) -> HttpServiceBuilder
where
- O: FnOnce(&h2::Config) -> R,
- {
- let _ = f(&self.config.h2config);
- self
- }
-
- /// Provide service for `EXPECT: 100-Continue` support.
- ///
- /// Service get called with request that contains `EXPECT` header.
- /// Service must return request in case of success, in that case
- /// request will be forwarded to main service.
- pub fn expect(self, expect: XF) -> HttpServiceBuilder
- where
- XF: IntoServiceFactory,
- X1: ServiceFactory,
- X1::InitError: fmt::Debug,
+ CF: IntoServiceFactory>,
+ CT: ServiceFactory, Response = h1::ControlAck>,
+ CT::Error: Error,
+ CT::InitError: fmt::Debug,
{
HttpServiceBuilder {
config: self.config,
- expect: expect.into_factory(),
- upgrade: self.upgrade,
- on_request: self.on_request,
+ h2_control: self.h2_control,
+ h1_control: control.into_factory(),
_t: PhantomData,
}
}
- /// Provide service for custom `Connection: UPGRADE` support.
- ///
- /// If service is provided then normal requests handling get halted
- /// and this service get called with original request and framed object.
- pub fn upgrade(self, upgrade: UF) -> HttpServiceBuilder
- where
- UF: IntoServiceFactory, Codec)>,
- U1: ServiceFactory<(Request, Io, Codec), Response = ()>,
- U1::Error: fmt::Display + Error,
- U1::InitError: fmt::Debug,
- {
- HttpServiceBuilder {
- config: self.config,
- expect: self.expect,
- upgrade: Some(upgrade.into_factory()),
- on_request: self.on_request,
- _t: PhantomData,
- }
- }
-
- /// Set req request callback.
- ///
- /// It get called once per request.
- pub fn on_request(mut self, f: FR) -> Self
- where
- FR: IntoService,
- R: Service<(Request, IoRef), Response = Request, Error = Response> + 'static,
- {
- self.on_request = Some(boxed::service(f.into_service()));
- self
- }
-
/// Finish service configuration and create *http service* for HTTP/1 protocol.
- pub fn h1(self, service: SF) -> H1Service
+ pub fn h1(self, service: SF) -> H1Service
where
B: MessageBody,
SF: IntoServiceFactory,
@@ -209,14 +161,36 @@ where
S::InitError: fmt::Debug,
S::Response: Into>,
{
- H1Service::with_config(self.config, service.into_factory())
- .expect(self.expect)
- .upgrade(self.upgrade)
- .on_request(self.on_request)
+ H1Service::with_config(self.config, service.into_factory()).control(self.h1_control)
+ }
+
+ /// Provide control service for http/2 protocol.
+ pub fn h2_control(self, control: CF) -> HttpServiceBuilder
+ where
+ CF: IntoServiceFactory>,
+ CT: ServiceFactory, Response = h2::ControlResult>,
+ CT::Error: Error,
+ CT::InitError: fmt::Debug,
+ {
+ HttpServiceBuilder {
+ config: self.config,
+ h1_control: self.h1_control,
+ h2_control: control.into_factory(),
+ _t: PhantomData,
+ }
+ }
+
+ /// Configure http2 connection settings
+ pub fn h2_configure(self, f: O) -> Self
+ where
+ O: FnOnce(&h2::Config) -> R,
+ {
+ let _ = f(&self.config.h2config);
+ self
}
/// Finish service configuration and create *http service* for HTTP/2 protocol.
- pub fn h2(self, service: SF) -> H2Service
+ pub fn h2(self, service: SF) -> H2Service
where
B: MessageBody + 'static,
SF: IntoServiceFactory,
@@ -224,11 +198,11 @@ where
S::InitError: fmt::Debug,
S::Response: Into> + 'static,
{
- H2Service::with_config(self.config, service.into_factory())
+ H2Service::with_config(self.config, service.into_factory()).control(self.h2_control)
}
/// Finish service configuration and create `HttpService` instance.
- pub fn finish(self, service: SF) -> HttpService
+ pub fn finish(self, service: SF) -> HttpService
where
B: MessageBody + 'static,
SF: IntoServiceFactory,
@@ -237,8 +211,7 @@ where
S::Response: Into> + 'static,
{
HttpService::with_config(self.config, service.into_factory())
- .expect(self.expect)
- .upgrade(self.upgrade)
- .on_request(self.on_request)
+ .h1_control(self.h1_control)
+ .h2_control(self.h2_control)
}
}
diff --git a/ntex/src/http/client/error.rs b/ntex/src/http/client/error.rs
index c097a25e..e7ef3119 100644
--- a/ntex/src/http/client/error.rs
+++ b/ntex/src/http/client/error.rs
@@ -7,7 +7,7 @@ use thiserror::Error;
#[cfg(feature = "openssl")]
use crate::connect::openssl::{HandshakeError, SslError};
-use crate::http::error::{HttpError, ParseError, PayloadError};
+use crate::http::error::{DecodeError, EncodeError, HttpError, PayloadError};
use crate::util::Either;
/// A set of errors that can occur during parsing json payloads
@@ -142,9 +142,12 @@ pub enum SendRequestError {
/// Error sending request
#[error("Error sending request: {0}")]
Send(#[from] io::Error),
+ /// Error encoding request
+ #[error("Error during request encoding: {0}")]
+ Request(#[from] EncodeError),
/// Error parsing response
#[error("Error during response parsing: {0}")]
- Response(#[from] ParseError),
+ Response(#[from] DecodeError),
/// Http error
#[error("{0}")]
Http(#[from] HttpError),
@@ -162,17 +165,17 @@ pub enum SendRequestError {
Error(#[from] Box),
}
-impl From> for SendRequestError {
- fn from(err: Either) -> Self {
+impl From> for SendRequestError {
+ fn from(err: Either) -> Self {
match err {
- Either::Left(err) => SendRequestError::Send(err),
+ Either::Left(err) => SendRequestError::Request(err),
Either::Right(err) => SendRequestError::Send(err),
}
}
}
-impl From> for SendRequestError {
- fn from(err: Either) -> Self {
+impl From> for SendRequestError {
+ fn from(err: Either) -> Self {
match err {
Either::Left(err) => SendRequestError::Response(err),
Either::Right(err) => SendRequestError::Send(err),
diff --git a/ntex/src/http/config.rs b/ntex/src/http/config.rs
index c5a3e65e..ce7d9dde 100644
--- a/ntex/src/http/config.rs
+++ b/ntex/src/http/config.rs
@@ -2,10 +2,8 @@ use std::{cell::Cell, ptr::copy_nonoverlapping, rc::Rc, time};
use ntex_h2::{self as h2};
-use crate::http::{Request, Response};
-use crate::service::{boxed::BoxService, Pipeline};
use crate::time::{sleep, Millis, Seconds};
-use crate::{io::IoRef, util::BytesMut};
+use crate::{service::Pipeline, util::BytesMut};
#[derive(Debug, PartialEq, Eq, Clone, Copy)]
/// Server keep-alive setting
@@ -236,12 +234,9 @@ impl ServiceConfig {
}
}
-pub(super) type OnRequest = BoxService<(Request, IoRef), Request, Response>;
-
-pub(super) struct DispatcherConfig {
+pub(super) struct DispatcherConfig {
pub(super) service: Pipeline,
- pub(super) expect: Pipeline,
- pub(super) upgrade: Option>,
+ pub(super) control: Pipeline,
pub(super) keep_alive: Seconds,
pub(super) client_disconnect: Seconds,
pub(super) h2config: h2::Config,
@@ -249,22 +244,13 @@ pub(super) struct DispatcherConfig {
pub(super) headers_read_rate: Option,
pub(super) payload_read_rate: Option,
pub(super) timer: DateService,
- pub(super) on_request: Option>,
}
-impl DispatcherConfig {
- pub(super) fn new(
- cfg: ServiceConfig,
- service: S,
- expect: X,
- upgrade: Option,
- on_request: Option,
- ) -> Self {
+impl DispatcherConfig {
+ pub(super) fn new(cfg: ServiceConfig, service: S, control: C) -> Self {
DispatcherConfig {
service: service.into(),
- expect: expect.into(),
- upgrade: upgrade.map(|v| v.into()),
- on_request: on_request.map(|v| v.into()),
+ control: control.into(),
keep_alive: cfg.keep_alive,
client_disconnect: cfg.client_disconnect,
ka_enabled: cfg.ka_enabled,
diff --git a/ntex/src/http/h1/client.rs b/ntex/src/http/h1/client.rs
index d1f11b5e..b222d178 100644
--- a/ntex/src/http/h1/client.rs
+++ b/ntex/src/http/h1/client.rs
@@ -1,11 +1,11 @@
-use std::{cell::Cell, cell::RefCell, io};
+use std::{cell::Cell, cell::RefCell};
use bitflags::bitflags;
use crate::codec::{Decoder, Encoder};
use crate::http::body::BodySize;
use crate::http::config::DateService;
-use crate::http::error::{ParseError, PayloadError};
+use crate::http::error::{DecodeError, EncodeError, PayloadError};
use crate::http::message::{ConnectionType, RequestHeadType, ResponseHead};
use crate::http::{Method, Version};
use crate::util::{Bytes, BytesMut};
@@ -117,7 +117,7 @@ impl ClientPayloadCodec {
impl Decoder for ClientCodec {
type Item = ResponseHead;
- type Error = ParseError;
+ type Error = DecodeError;
fn decode(&self, src: &mut BytesMut) -> Result
diff --git a/ntex/src/http/error.rs b/ntex/src/http/error.rs
index f32fcdde..f13552bf 100644
--- a/ntex/src/http/error.rs
+++ b/ntex/src/http/error.rs
@@ -57,9 +57,24 @@ impl ResponseError for io::Error {}
/// `InternalServerError` for `JsonError`
impl ResponseError for serde_json::error::Error {}
+/// A set of errors that can occur during HTTP streams encoding
+#[derive(thiserror::Error, Debug)]
+pub enum EncodeError {
+ /// An invalid `HttpVersion`, such as `HTP/1.1`
+ #[error("Unsupported HTTP version specified")]
+ UnsupportedVersion(super::Version),
+
+ #[error("Unexpected end of bytes stream")]
+ UnexpectedEof,
+
+ /// Internal error
+ #[error("Internal error")]
+ Internal(Box),
+}
+
/// A set of errors that can occur during parsing HTTP streams
#[derive(thiserror::Error, Debug)]
-pub enum ParseError {
+pub enum DecodeError {
/// An invalid `Method`, such as `GE.T`.
#[error("Invalid Method specified")]
Method,
@@ -74,17 +89,13 @@ pub enum ParseError {
Header,
/// A message head is too large to be reasonable.
#[error("Message head is too large")]
- TooLarge,
+ TooLarge(usize),
/// A message reached EOF, but is not complete.
#[error("Message is incomplete")]
Incomplete,
/// An invalid `Status`, such as `1337 ELITE`.
#[error("Invalid Status provided")]
Status,
- /// A timeout occurred waiting for an IO event.
- #[allow(dead_code)]
- #[error("Timeout during parse")]
- Timeout,
/// An `InvalidInput` occurred while trying to parse incoming stream.
#[error("`InvalidInput` occurred while trying to parse incoming stream: {0}")]
InvalidInput(&'static str),
@@ -93,22 +104,22 @@ pub enum ParseError {
Utf8(#[from] Utf8Error),
}
-impl From for ParseError {
- fn from(err: FromUtf8Error) -> ParseError {
- ParseError::Utf8(err.utf8_error())
+impl From for DecodeError {
+ fn from(err: FromUtf8Error) -> DecodeError {
+ DecodeError::Utf8(err.utf8_error())
}
}
-impl From for ParseError {
- fn from(err: httparse::Error) -> ParseError {
+impl From for DecodeError {
+ fn from(err: httparse::Error) -> DecodeError {
match err {
httparse::Error::HeaderName
| httparse::Error::HeaderValue
| httparse::Error::NewLine
- | httparse::Error::Token => ParseError::Header,
- httparse::Error::Status => ParseError::Status,
- httparse::Error::TooManyHeaders => ParseError::TooLarge,
- httparse::Error::Version => ParseError::Version,
+ | httparse::Error::Token => DecodeError::Header,
+ httparse::Error::Status => DecodeError::Status,
+ httparse::Error::TooManyHeaders => DecodeError::TooLarge(0),
+ httparse::Error::Version => DecodeError::Version,
}
}
}
@@ -131,9 +142,9 @@ pub enum PayloadError {
/// Http2 payload error
#[error("{0}")]
Http2Payload(#[from] h2::StreamError),
- /// Parse error
- #[error("Parse error: {0}")]
- Parse(#[from] ParseError),
+ /// Decode error
+ #[error("Decode error: {0}")]
+ Decode(#[from] DecodeError),
/// Io error
#[error("{0}")]
Io(#[from] io::Error),
@@ -153,61 +164,11 @@ impl From> for PayloadError {
pub enum DispatchError {
/// Service error
#[error("Service error")]
- Service(Box),
+ Service(Box),
- /// Upgrade service error
- #[error("Upgrade service error: {0}")]
- Upgrade(Box),
-
- /// Peer is disconnected, error indicates that peer is disconnected because of it
- #[error("Disconnected: {0:?}")]
- PeerGone(Option),
-
- /// Http request parse error.
- #[error("Parse error: {0}")]
- Parse(#[from] ParseError),
-
- /// Http response encoding error.
- #[error("Encode error: {0}")]
- Encode(io::Error),
-
- /// Http/2 error
- #[error("{0}")]
- H2(#[from] H2Error),
-
- /// The first request did not complete within the specified timeout.
- #[error("The first request did not complete within the specified timeout")]
- SlowRequestTimeout,
-
- /// Disconnect timeout. Makes sense for ssl streams.
- #[error("Connection shutdown timeout")]
- DisconnectTimeout,
-
- /// Payload is not consumed
- #[error("Task is completed but request's payload is not consumed")]
- PayloadIsNotConsumed,
-
- /// Malformed request
- #[error("Malformed request")]
- MalformedRequest,
-
- /// Response body processing error
- #[error("Response body processing error: {0}")]
- ResponsePayload(Box),
-
- /// Internal error
- #[error("Internal error")]
- InternalError,
-
- /// Unknown error
- #[error("Unknown error")]
- Unknown,
-}
-
-impl From for DispatchError {
- fn from(err: io::Error) -> Self {
- DispatchError::PeerGone(Some(err))
- }
+ /// Control service error
+ #[error("Control service error: {0}")]
+ Control(Box),
}
#[derive(thiserror::Error, Debug)]
@@ -296,15 +257,16 @@ mod tests {
#[test]
fn test_payload_error() {
- let err: PayloadError = io::Error::new(io::ErrorKind::Other, "ParseError").into();
- assert!(format!("{}", err).contains("ParseError"));
+ let err: PayloadError = io::Error::new(io::ErrorKind::Other, "DecodeError").into();
+ assert!(format!("{}", err).contains("DecodeError"));
let err: PayloadError = BlockingError::Canceled.into();
assert!(format!("{}", err).contains("Operation is canceled"));
let err: PayloadError =
- BlockingError::Error(io::Error::new(io::ErrorKind::Other, "ParseError")).into();
- assert!(format!("{}", err).contains("ParseError"));
+ BlockingError::Error(io::Error::new(io::ErrorKind::Other, "DecodeError"))
+ .into();
+ assert!(format!("{}", err).contains("DecodeError"));
let err = PayloadError::Incomplete(None);
assert_eq!(
@@ -315,7 +277,7 @@ mod tests {
macro_rules! from {
($from:expr => $error:pat) => {
- match ParseError::from($from) {
+ match DecodeError::from($from) {
e @ $error => {
assert!(format!("{}", e).len() >= 5);
}
@@ -326,13 +288,13 @@ mod tests {
#[test]
fn test_from() {
- from!(httparse::Error::HeaderName => ParseError::Header);
- from!(httparse::Error::HeaderName => ParseError::Header);
- from!(httparse::Error::HeaderValue => ParseError::Header);
- from!(httparse::Error::NewLine => ParseError::Header);
- from!(httparse::Error::Status => ParseError::Status);
- from!(httparse::Error::Token => ParseError::Header);
- from!(httparse::Error::TooManyHeaders => ParseError::TooLarge);
- from!(httparse::Error::Version => ParseError::Version);
+ from!(httparse::Error::HeaderName => DecodeError::Header);
+ from!(httparse::Error::HeaderName => DecodeError::Header);
+ from!(httparse::Error::HeaderValue => DecodeError::Header);
+ from!(httparse::Error::NewLine => DecodeError::Header);
+ from!(httparse::Error::Status => DecodeError::Status);
+ from!(httparse::Error::Token => DecodeError::Header);
+ from!(httparse::Error::TooManyHeaders => DecodeError::TooLarge(0));
+ from!(httparse::Error::Version => DecodeError::Version);
}
}